module LanguageServer
  module Protocol
    module Interface
<% each do |name| -%>
      autoload :<%= name %>, "language_server/protocol/interface/<%= name.underscore %>"
<% end -%>

<% each do |name| -%>
      require_relative "interface/<%= name.underscore %>"
<% end -%>
    end
  end
end
