module LanguageServer
  module Protocol
    module Constant
      #
      # Inlay hint kinds.
      #
      module InlayHintKind
        #
        # An inlay hint that for a type annotation.
        #
        TYPE = 1
        #
        # An inlay hint that is for a parameter.
        #
        PARAMETER = 2
      end
    end
  end
end
