module LanguageServer
  module Protocol
    module Constant
      module TraceValues
        #
        # Turn tracing off.
        #
        OFF = "off"
        #
        # Trace messages only.
        #
        MESSAGES = "messages"
        #
        # Verbose message tracing.
        #
        VERBOSE = "verbose"
      end
    end
  end
end
