module LanguageServer
  module Protocol
    module Constant
      module WatchKind
        #
        # Interested in create events.
        #
        CREATE = 1
        #
        # Interested in change events
        #
        CHANGE = 2
        #
        # Interested in delete events
        #
        DELETE = 4
      end
    end
  end
end
